Attribute VB_Name = "Main"
Option Explicit
Option Private Module

Public Sub RunAddWorker()
  Dim iWorkers As DB_Workers: Set iWorkers = AccessWorkers
  Dim iInput As ItemInputWorker: Set iInput = PromptInputWorker(iWorkers)
  If iInput Is Nothing Then _
    Exit Sub
  
  Call iWorkers.AddWroker(iInput.workerID_, iInput.workerName_, iInput.path_)
  Call ThisWorkbook.Sheets(SHEET_WORKERS).Activate
  Call UserInteraction.ShowMessage(IM_ADD_WORKER_OK, iInput.workerName_)
End Sub

Public Sub RunReloadWorker()
  Dim iWorkers As DB_Workers: Set iWorkers = AccessWorkers
  Dim sInputID$: sInputID = PromptWorkerID(iWorkers)
  If sInputID = vbNullString Then _
    Exit Sub
  
  Dim xlApp As New API_XLWrapper: Call xlApp.SetApplication(ThisWorkbook.Application)
  Call xlApp.PauseUI
  Dim bLoaded As Boolean: bLoaded = ReloadWorker(iWorkers.Access(sInputID), AccessGlobalWorks, GetStartDate)
  Call xlApp.ResumeUI
  
  If bLoaded Then _
    Call UserInteraction.ShowMessage(IM_RELOAD_WORKER_OK)
End Sub

Public Sub RunRemoveWorker()
  Dim iWorkers As DB_Workers: Set iWorkers = AccessWorkers
  Dim sInputID$: sInputID = PromptWorkerID(iWorkers)
  If sInputID = vbNullString Then _
    Exit Sub
  
  Dim iWorker As IteratorWorker: Set iWorker = iWorkers.Access(sInputID)
  If Not iWorker.Active Then
    Call UserInteraction.ShowMessage(EM_WORKER_ALREADY_REMOVED, iWorker.WorkerName)
  Else
    iWorker.Active = False
    Call ThisWorkbook.Sheets(SHEET_WORKERS).Activate
    Call UserInteraction.ShowMessage(IM_REMOVE_WORKER_OK, iWorker.WorkerName)
  End If
End Sub

Public Sub RunReloadAll()
  Dim iWorkers As DB_Workers: Set iWorkers = AccessWorkers
  Dim nStartDate&: nStartDate = GetStartDate
  Dim iGlobalWorks As DB_GlobalWorks: Set iGlobalWorks = AccessGlobalWorks
  
  Call CSE_ProgressBar.Init(" ", maxVal:=iWorkers.CountActive)
  Call CSE_ProgressBar.ShowModeless
  
  Dim xlApp As New API_XLWrapper: Call xlApp.SetApplication(ThisWorkbook.Application)
  Call xlApp.PauseUI
  
  Dim iWorker As IteratorWorker: Set iWorker = iWorkers.IBegin
  Do While Not iWorker.IsDone
    If iWorker.Active Then
      CSE_ProgressBar.Description = iWorker.WorkerName
      Call ReloadWorker(iWorker, iGlobalWorks, nStartDate)
      Call CSE_ProgressBar.IncrementA
    End If
    Call iWorker.Increment
  Loop
  
  Call UpdatePivots
  Call xlApp.ResumeUI
  
  Call Unload(CSE_ProgressBar)
  Call UserInteraction.ShowMessage(IM_RELOAD_ALL_OK)
End Sub

Public Sub RunInfoProjects()
  Dim xlApp As New API_XLWrapper: Call xlApp.SetApplication(ThisWorkbook.Application)
  Dim iProjects As Excel.Workbook: Set iProjects = xlApp.OpenDocument(PATH_PROJECTS, bReadOnly:=True)
  If iProjects Is Nothing Then
    Call UserInteraction.ShowMessage(EM_CANNOT_ACCESS_PROJECTS, PATH_PROJECTS)
    Exit Sub
  End If
End Sub

Public Sub RunInfoCalendar()
  Call UserInteraction.FollowHyperlink(ActiveWorkbook, URL_CALENDAR_CONSULTANT & "/" & VBA.Year(VBA.Int(VBA.Now)))
End Sub
