VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "DB_CachedProjects"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public projects_ As Scripting.Dictionary
Private substitutes_ As Scripting.Dictionary

Public Function Reload(iSource As Excel.Workbook)
  Set projects_ = New Scripting.Dictionary
  Set substitutes_ = New Scripting.Dictionary
  Call ScanSubsitutes(iSource)
  Call ScanActiveProjects(iSource)
  Call ScanArchiveProjects(iSource)
End Function

Public Function GetProjectCategory(sProject$) As TProjectCategory
  Dim sFixedProject$: sFixedProject = ApplySustitutions(sProject)
  If Not projects_.Exists(sFixedProject) Then
    GetProjectCategory = T_PCAT_ERR
  Else
    GetProjectCategory = projects_(sFixedProject).category_
  End If
End Function

Public Function GetProjectManager(sProject$) As String
  Dim sFixedProject$: sFixedProject = ApplySustitutions(sProject)
  If Not projects_.Exists(sFixedProject) Then
    Exit Function
  Else
    GetProjectManager = projects_(sFixedProject).manager_
  End If
End Function

' =======
Private Function ApplySustitutions(sProject$) As String
  If substitutes_.Exists(sProject) Then
    ApplySustitutions = substitutes_(sProject)
  Else
    ApplySustitutions = sProject
  End If
End Function

Private Function ScanActiveProjects(iSource As Excel.Workbook)
  Dim iData() As String: iData = LoadActiveProjects(iSource)
  If ArraySize(iData, 1) < 1 Then _
    Exit Function
  Dim nItem&
  Dim iProject As ItemCachedProject
  For nItem = LBound(iData, 1) To UBound(iData, 1) Step 1
    Set iProject = New ItemCachedProject
    Dim sProject$: sProject = iData(nItem, S_TP_NAME)
    With iProject
      .isActive_ = True
      .category_ = iData(nItem, S_TP_TYPE)
      .name_ = sProject
      .manager_ = iData(nItem, S_TP_MANAGER)
    End With
    If Not projects_.Exists(sProject) Then
      Call projects_.Add(sProject, iProject.Clone())
    Else
      Debug.Print sProject
    End If
    
  Next nItem
End Function

Private Function ScanArchiveProjects(iSource As Excel.Workbook)
  Dim iData() As String: iData = LoadArchiveProjects(iSource)
  If ArraySize(iData, 1) < 1 Then _
    Exit Function
  Dim nItem&
  Dim iProject As ItemCachedProject
  For nItem = LBound(iData, 1) To UBound(iData, 1) Step 1
    Set iProject = New ItemCachedProject
    Dim sProject$: sProject = iData(nItem, S_TP_NAME)
    With iProject
      .isActive_ = False
      .category_ = iData(nItem, S_TP_TYPE)
      .name_ = sProject
      .manager_ = iData(nItem, S_TP_MANAGER)
    End With
    If Not projects_.Exists(sProject) Then
      Call projects_.Add(sProject, iProject.Clone())
    Else
      Debug.Print sProject
    End If
  Next nItem
End Function

Private Function ScanSubsitutes(iSource As Excel.Workbook)
  Dim iData() As String: iData = LoadProjectSubstitutes(iSource)
  If ArraySize(iData, 1) < 1 Then _
    Exit Function
  Dim nItem&
  For nItem = LBound(iData, 1) To UBound(iData, 1) Step 1
    Call substitutes_.Add(iData(nItem, S_TS_MATCH_TARGET), iData(nItem, S_TS_REPLACEMENT))
  Next nItem
End Function
