Attribute VB_Name = "Main"
'============       =================
Option Explicit
Option Private Module

Public Sub CR_AddYear()
  Dim iTarget As Excel.Workbook: Set iTarget = ActiveWorkbook
  Dim nYear%: nYear = PromptYear()
  If nYear = 0 Then _
    Exit Sub
  
  Dim xlApp As New API_XLWrapper: Call xlApp.SetApplication(ThisWorkbook.Application)
  Dim iSource As Excel.Workbook: Set iSource = xlApp.OpenDocument(PATH_CALENDAR, bReadOnly:=True)
  If iSource Is Nothing Then
    Call UserInteraction.ShowMessage(EM_ADD_YEAR_NO_ACCESS, PATH_CALENDAR)
    Exit Sub
  End If
  
  Dim iData As DB_WorkerData: Set iData = AccessWorker(iTarget)
  Dim bYearFound As Boolean: bYearFound = XLWorksheetExists(CStr(nYear), iSource)
    
  Call xlApp.PauseUI
  If bYearFound Then
    Call iData.ImportYear(iSource.Sheets(CStr(nYear)))
  Else
    Call iData.CreateYear(nYear)
  End If
  Call xlApp.ResumeUI
  Call xlApp.ReleaseDocument
  
  Call UserInteraction.ShowMessage(IIf(bYearFound, IM_ADD_YEAR_AUTO, IM_ADD_YEAR_MANUAL))
End Sub

Public Sub CR_AddWorkType()
  Dim target As Excel.Workbook: Set target = ActiveWorkbook
  Dim sWork$: sWork = PromptWorkTypeID(AccessWorkTypes(target))
  If sWork = vbNullString Then _
    Exit Sub
  
  target.Application.Selection = sWork
End Sub

Public Sub CR_AddActiveProject()
  Dim target As Excel.Workbook: Set target = ActiveWorkbook
  Dim sProject$: sProject = PromptActiveProject(AccessProjects)
  If sProject = vbNullString Then _
    Exit Sub
  
  target.Application.Selection = sProject
End Sub

Public Sub CR_InfoProjects()
  Dim xlApp As New API_XLWrapper: Call xlApp.SetApplication(ThisWorkbook.Application)
  Dim iProjects As Excel.Workbook: Set iProjects = xlApp.OpenDocument(PATH_PROJECTS, bReadOnly:=True)
  If iProjects Is Nothing Then
    Call UserInteraction.ShowMessage(EM_CANNOT_ACCESS_PROJECTS, PATH_PROJECTS)
    Exit Sub
  End If
End Sub

Public Sub CR_InfoCalendar()
  Call UserInteraction.FollowHyperlink(ActiveWorkbook, URL_CALENDAR_CONSULTANT & "/" & VBA.Year(VBA.Now))
End Sub

Public Sub CR_InfoExamples()
  Dim xlApp As New API_XLWrapper: Call xlApp.SetApplication(ThisWorkbook.Application)
  Dim iProjects As Excel.Workbook: Set iProjects = xlApp.OpenDocument(PATH_SAMPLES, bReadOnly:=True)
  If iProjects Is Nothing Then
    Call UserInteraction.ShowMessage(EM_CANNOT_ACCESS_SAMPLES, PATH_SAMPLES)
    Exit Sub
  End If
End Sub

Public Sub CR_DailyReport()
  Dim iTarget As Excel.Workbook: Set iTarget = ActiveWorkbook
  Dim targetDate&: targetDate = PromptDate(GetActiveDay(iTarget))
  If targetDate = 0 Then _
    Exit Sub
  
  Dim iData As DB_WorkerData: Set iData = AccessWorker(iTarget)
  Dim sDescription$: sDescription = GenerateDayDescription(targetDate, iData)
  If sDescription = vbNullString Then
    Call UserInteraction.ShowMessage(EM_DAILY_NO_WORKS)
    Exit Sub
  End If
  
  If Not CreateDailyMessage(targetDate, sDescription) Then _
    Call UserInteraction.ShowMessage(EM_CANNOT_CREATE_MAIL)
End Sub

Public Sub CR_MonthlyReport()
  Dim iTarget As Excel.Workbook: Set iTarget = ActiveWorkbook
  Dim targetDate&: targetDate = PromptDate(GetActiveDay(iTarget))
  If targetDate = 0 Then _
    Exit Sub
  
  Dim iData As DB_WorkerData: Set iData = AccessWorker(iTarget)
  Dim theWrap As New API_XLWrapper: Call theWrap.SetApplication(ThisWorkbook.Application)
  Dim iDoc As Excel.Workbook: Set iDoc = theWrap.NewDocument(PATH_MONTHLY_TEMPLATE)
  If iDoc Is Nothing Then
    Call UserInteraction.ShowMessage(EM_INVALID_REPORT_TEMPLATE)
    Exit Sub
  End If
  
  Dim iReport As New MonthlyReport: Call iReport.Init(iDoc, iData, AccessProjects)
  Call iReport.ImportData(VBA.Month(targetDate), VBA.Year(targetDate))
  
  Call UserInteraction.ShowMessage(IM_MONTH_REPORT_OK)
End Sub

Public Sub CR_UpdatePivot()
  Dim iTarget As Excel.Workbook: Set iTarget = ActiveWorkbook
  Dim iData As DB_WorkerData: Set iData = AccessWorker(iTarget)
  
  Call ReloadPivotData(iTarget.Sheets(SHEET_PIVOT_DATA), iData, AccessProjects)

  Dim aPivot As Excel.PivotTable
  Call iTarget.Sheets(SHEET_PIVOT).Activate
  For Each aPivot In iTarget.Sheets(SHEET_PIVOT).PivotTables
    With aPivot
      .RefreshTable
      .Update
    End With
  Next
  
  Call UserInteraction.ShowMessage(IM_UPDATE_PIVOT_OK)
End Sub

Public Sub CR_FixGroups()
  Call AccessWorker(ActiveWorkbook).FixDataGroups
End Sub

Public Sub CR_DeleteHolidays()
  Dim iData As DB_WorkerData: Set iData = AccessWorker(ActiveWorkbook)
  
  Dim xlUI As New API_XLWrapper: Call xlUI.SetApplication(ActiveWorkbook.Application)
  Call xlUI.PauseUI
  Dim nCount&: nCount = iData.DeleteHolidays
  Call xlUI.ResumeUI
  
  Call UserInteraction.ShowMessage(IM_HOLIDAYS_DELETED, nCount)
End Sub
