VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "MonthlyReport"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private source_ As DB_WorkerData
Private projects_ As DB_CachedProjects

Private out_ As Excel.Worksheet

Private start_ As Long
Private finish_ As Long
Private projectHours_ As Scripting.Dictionary
Private projectCategories_ As Scripting.Dictionary
Private hoursTotal_ As Double
Private hoursIncome_ As Double
Private hoursOutcome_ As Double

Private Enum ReportStruct
  S_MR_PROJECT = 2
  S_MR_WORK = 3
  S_MR_DESCRIPTION = 4
  S_MR_HOURS = 6
  S_MR_DATE = 7
  S_MR_PROXY = 10
End Enum

Private Const REPORT_SHEET_REPORT = ""

Private Const CELL_REPORT_WORKER_NAME = "c_WorkerName"
Private Const CELL_REPORT_PERIOD = "c_Period"
Private Const CELL_REPORT_INCOME = "c_Income"
Private Const CELL_REPORT_OUTCOME = "c_Outcome"
Private Const CELL_REPORT_TOTAL = "c_Total"

Private Const REPORT_ROW_PROTOTYPE = 6
Private Const REPORT_WORK_START = 7

Public Function Init(iOut As Excel.Workbook, iSource As DB_WorkerData, iProjects As DB_CachedProjects)
  Set out_ = iOut.Sheets(REPORT_SHEET_REPORT)
  Set source_ = iSource
  Set projects_ = iProjects
End Function

Public Function ImportData(nMonth%, nYear%)
  start_ = VBA.DateSerial(nYear, nMonth, 1)
  finish_ = VBA.DateSerial(nYear, nMonth + 1, 1) - 1
  
  Call FillHeader
  Call FillWorks
  Call FillTotals
  
  Dim r As Excel.Range
  Call out_.Parent.Names(CELL_REPORT_INCOME).RefersToRange.Select
End Function

' =======
Private Function FillHeader()
  out_.Parent.Names(CELL_REPORT_WORKER_NAME).RefersToRange = source_.WorkerName
  out_.Parent.Names(CELL_REPORT_PERIOD).RefersToRange = Fmt("   {1}  {2}", VBA.CDate(start_), VBA.CDate(finish_))
End Function

Private Function FillWorks()
  hoursTotal_ = 0
  hoursIncome_ = 0
  hoursOutcome_ = 0
  Set projectHours_ = New Scripting.Dictionary
  Set projectCategories_ = New Scripting.Dictionary
  
  Dim iWork As IteratorWork: Set iWork = source_.IBegin
  Call iWork.GoUntil(start_)
  
  Dim nRow&: nRow = REPORT_WORK_START
  Do While Not iWork.IsDone
    Dim theDate&: theDate = iWork.DDate
    If theDate > finish_ Then _
      Exit Do
    
    Dim dDuration As Double: dDuration = iWork.Duration
    If dDuration = 0 Then _
      GoTo NEXT_WORK
      
    Dim sProject$: sProject = iWork.Project
    If sProject = PROJECT_HOLIDAYS Then _
      GoTo NEXT_WORK
    
    Call AddWork(nRow, theDate, sProject, iWork.WorkType, dDuration, iWork.Description, iWork.IsMergedDescription)
    
    nRow = nRow + 1
    Call IncrementHours(sProject, dDuration)
    
NEXT_WORK:
    Call iWork.Increment
  Loop
  
  Call out_.Rows(nRow).Delete
  Call out_.Rows(REPORT_ROW_PROTOTYPE).Delete
End Function

Private Function FillTotals()
  Dim nIncomeHead&: nIncomeHead = out_.Parent.Names(CELL_REPORT_INCOME).RefersToRange.Row
  Dim nOutcomeHead&: nOutcomeHead = out_.Parent.Names(CELL_REPORT_OUTCOME).RefersToRange.Row
  out_.Parent.Names(CELL_REPORT_TOTAL).RefersToRange = hoursTotal_
  Call SetTotalHours(nIncomeHead, hoursIncome_)
  Call SetTotalHours(nOutcomeHead, hoursOutcome_)
  
  Dim nIncome&: nIncome = nIncomeHead + 2
  Dim nOutcome&: nOutcome = nOutcomeHead + 2
  Dim nPrototype&: nPrototype = nIncomeHead + 1
  
  Dim aKey As Variant
  For Each aKey In projectCategories_
    Dim sProject$: sProject = CStr(aKey)
    Dim sManager$: sManager = projects_.GetProjectManager(sProject)
    Dim dDuration As Double: dDuration = projectHours_(aKey)
    If projectCategories_(aKey) = T_PCAT_INCOME Then
      Call AddProject(nPrototype, nIncome, sProject, sManager, dDuration)
      nIncome = nIncome + 1
      nOutcomeHead = nOutcomeHead + 1
      nOutcome = nOutcome + 1
    Else
      Call AddProject(nPrototype, nOutcome, sProject, sManager, dDuration)
      nOutcome = nOutcome + 1
    End If
  Next aKey
  
  Call out_.Rows(nOutcome).Delete
  Call out_.Rows(nOutcomeHead + 1).Delete
  Call out_.Rows(nIncome).Delete
  Call out_.Rows(nIncomeHead + 1).Delete
End Function

' =======
Private Function AddWork(nRow&, theDate&, sProject$, sWork$, dDuration As Double, sDescription$, bIsMerged As Boolean)
  Call out_.Rows(REPORT_ROW_PROTOTYPE).EntireRow.Copy
  Call out_.Rows(nRow).EntireRow.Insert
  out_.Cells(nRow, S_MR_PROJECT) = sProject
  out_.Cells(nRow, S_MR_WORK) = sWork
  out_.Cells(nRow, S_MR_HOURS) = dDuration
  out_.Cells(nRow, S_MR_DATE) = theDate
  If bIsMerged Then
    Call AddMergedRow(nRow)
  Else
    out_.Cells(nRow, S_MR_DESCRIPTION) = sDescription
  End If
End Function

Private Function AddProject(nPrototype&, nRow&, sProject$, sManager$, dDuration As Double)
  Call out_.Rows(nPrototype).EntireRow.Copy
  Call out_.Rows(nRow).EntireRow.Insert
  out_.Cells(nRow, S_MR_PROJECT) = sProject
  out_.Cells(nRow, S_MR_WORK) = sManager
  Call SetTotalHours(nRow, dDuration)
End Function

Private Function AddMergedRow(nRow&)
  Dim iPrevious As Excel.Range: Set iPrevious = out_.Cells(nRow - 1, S_MR_DESCRIPTION)
  Dim nCount&
  If iPrevious.MergeCells Then
    nCount = iPrevious.MergeArea.Rows.Count
  Else
    nCount = 1
  End If
  
  out_.Cells(nRow, S_MR_PROXY) = ""
  out_.Cells(nRow - nCount, S_MR_PROXY) = ""

  Dim iMergeArea As Excel.Range
  Set iMergeArea = out_.Cells(nRow, S_MR_DESCRIPTION).Offset(-nCount, 0).Resize(nCount + 1, 2)
  Call iMergeArea.UnMerge
  Call iMergeArea.Merge
End Function

Private Function IncrementHours(sProject$, dDuration As Double)
  If projectHours_.Exists(sProject) Then
    projectHours_(sProject) = projectHours_(sProject) + dDuration
  Else
    Call projectHours_.Add(sProject, dDuration)
    Call projectCategories_.Add(sProject, projects_.GetProjectCategory(sProject))
  End If
  hoursTotal_ = hoursTotal_ + dDuration
  
  If projectCategories_(sProject) = T_PCAT_INCOME Then
    hoursIncome_ = hoursIncome_ + dDuration
  Else
    hoursOutcome_ = hoursOutcome_ + dDuration
  End If
End Function

Private Function SetTotalHours(nRow&, dDuration As Double)
  out_.Cells(nRow, S_MR_HOURS) = dDuration
  out_.Cells(nRow, S_MR_DATE) = dDuration / hoursTotal_
End Function
